@@ -32,29 +32,29 @@ class RPiCamStillApp : public RPiCamApp
 public:
 	RPiCamStillApp() : RPiCamApp(std::make_unique<StillOptions>()) {}
 
-	StillOptions *GetOptions() const { return static_cast<StillOptions *>(options_.get()); }
+	StillOptions *GetOptions() const { return static_cast<StillOptions *>(RPiCamApp::GetOptions()); }
 };
 
 static std::string generate_filename(StillOptions const *options)
 {
 	char filename[128];
-	std::string folder = options->output; // sometimes "output" is used as a folder name
+	std::string folder = options->Get().output; // sometimes "output" is used as a folder name
 	if (!folder.empty() && folder.back() != '/')
 		folder += "/";
-	if (options->datetime)
+	if (options->Get().datetime)
 	{
 		std::time_t raw_time;
 		std::time(&raw_time);
 		char time_string[32];
 		std::tm *time_info = std::localtime(&raw_time);
 		std::strftime(time_string, sizeof(time_string), "%m%d%H%M%S", time_info);
-		snprintf(filename, sizeof(filename), "%s%s.%s", folder.c_str(), time_string, options->encoding.c_str());
+		snprintf(filename, sizeof(filename), "%s%s.%s", folder.c_str(), time_string, options->Get().encoding.c_str());
 	}
-	else if (options->timestamp)
+	else if (options->Get().timestamp)
 		snprintf(filename, sizeof(filename), "%s%u.%s", folder.c_str(), (unsigned)time(NULL),
-				 options->encoding.c_str());
+				 options->Get().encoding.c_str());
 	else
-		snprintf(filename, sizeof(filename), options->output.c_str(), options->framestart);
+		snprintf(filename, sizeof(filename), options->Get().output.c_str(), options->Get().framestart);
 	filename[sizeof(filename) - 1] = 0;
 	return std::string(filename);
 }
@@ -62,17 +62,17 @@ static std::string generate_filename(StillOptions const *options)
 static void update_latest_link(std::string const &filename, StillOptions const *options)
 {
 	// Create a fixed-name link to the most recent output file, if requested.
-	if (!options->latest.empty())
+	if (!options->Get().latest.empty())
 	{
-		fs::path link { options->latest };
+		fs::path link { options->Get().latest };
 		fs::path target { filename };
 
 		if (fs::exists(link) && !fs::remove(link))
-			LOG_ERROR("WARNING: could not delete latest link " << options->latest);
+			LOG_ERROR("WARNING: could not delete latest link " << options->Get().latest);
 		else
 		{
 			fs::create_symlink(target, link);
-			LOG(2, "Link " << options->latest << " created");
+			LOG(2, "Link " << options->Get().latest << " created");
 		}
 	}
 }
@@ -86,11 +86,11 @@ static void save_image(RPiCamStillApp &app, CompletedRequestPtr &payload, Stream
 	const std::vector<libcamera::Span<uint8_t>> mem = r.Get();
 	if (stream == app.RawStream())
 		dng_save(mem, info, payload->metadata, filename, app.CameraModel(), options);
-	else if (options->encoding == "jpg")
+	else if (options->Get().encoding == "jpg")
 		jpeg_save(mem, info, payload->metadata, filename, app.CameraModel(), options);
-	else if (options->encoding == "png")
+	else if (options->Get().encoding == "png")
 		png_save(mem, info, filename, options);
-	else if (options->encoding == "bmp")
+	else if (options->Get().encoding == "bmp")
 		bmp_save(mem, info, filename, options);
 	else
 		yuv_save(mem, info, filename, options);
@@ -103,21 +103,21 @@ static void save_images(RPiCamStillApp &app, CompletedRequestPtr &payload)
 	std::string filename = generate_filename(options);
 	save_image(app, payload, app.StillStream(), filename);
 	update_latest_link(filename, options);
-	if (options->raw)
+	if (options->Get().raw)
 	{
 		filename = filename.substr(0, filename.rfind('.')) + ".dng";
 		save_image(app, payload, app.RawStream(), filename);
 	}
-	options->framestart++;
-	if (options->wrap)
-		options->framestart %= options->wrap;
+	options->Set().framestart++;
+	if (options->Get().wrap)
+		options->Set().framestart %= options->Get().wrap;
 }
 
 static void save_metadata(StillOptions const *options, libcamera::ControlList &metadata)
 {
 	std::streambuf *buf = std::cout.rdbuf();
 	std::ofstream of;
-	const std::string &filename = options->metadata;
+	const std::string &filename = options->Get().metadata;
 
 	if (filename.compare("-"))
 	{
@@ -125,7 +125,7 @@ static void save_metadata(StillOptions const *options, libcamera::ControlList &m
 		buf = of.rdbuf();
 	}
 
-	write_metadata(buf, options->metadata_format, metadata, true);
+	write_metadata(buf, options->Get().metadata_format, metadata, true);
 }
 
 // Some keypress/signal handling.
@@ -139,7 +139,7 @@ static void default_signal_handler(int signal_number)
 static int get_key_or_signal(StillOptions const *options, pollfd p[1])
 {
 	int key = 0;
-	if (options->keypress)
+	if (options->Get().keypress)
 	{
 		poll(p, 1, 0);
 		if (p[0].revents & POLLIN)
@@ -150,7 +150,7 @@ static int get_key_or_signal(StillOptions const *options, pollfd p[1])
 			key = user_string[0];
 		}
 	}
-	if (options->signal)
+	if (options->Get().signal)
 	{
 		if (signal_received == SIGUSR1)
 			key = '\n';
@@ -166,16 +166,18 @@ static int get_key_or_signal(StillOptions const *options, pollfd p[1])
 static void event_loop(RPiCamStillApp &app)
 {
 	StillOptions const *options = app.GetOptions();
-	bool output = !options->output.empty() || options->datetime || options->timestamp; // output requested?
-	bool keypress = options->keypress || options->signal; // "signal" mode is much like "keypress" mode
+	// output requested?
+	bool output = !options->Get().output.empty() || options->Get().datetime || options->Get().timestamp;
+	// "signal" mode is much like "keypress" mode
+	bool keypress = options->Get().keypress || options->Get().signal;
 	unsigned int still_flags = RPiCamApp::FLAG_STILL_NONE;
-	if (options->encoding == "rgb24" || options->encoding == "png")
+	if (options->Get().encoding == "rgb24" || options->Get().encoding == "png")
 		still_flags |= RPiCamApp::FLAG_STILL_BGR;
-	if (options->encoding == "rgb48")
+	if (options->Get().encoding == "rgb48")
 		still_flags |= RPiCamApp::FLAG_STILL_BGR48;
-	else if (options->encoding == "bmp")
+	else if (options->Get().encoding == "bmp")
 		still_flags |= RPiCamApp::FLAG_STILL_RGB;
-	if (options->raw)
+	if (options->Get().raw)
 		still_flags |= RPiCamApp::FLAG_STILL_RAW;
 
 	app.OpenCamera();
@@ -185,7 +187,7 @@ static void event_loop(RPiCamStillApp &app)
 	signal(SIGUSR2, default_signal_handler);
 	pollfd p[1] = { { STDIN_FILENO, POLLIN, 0 } };
 
-	if (options->immediate)
+	if (options->Get().immediate)
 	{
 		app.ConfigureStill(still_flags);
 		while (keypress)
@@ -198,7 +200,7 @@ static void event_loop(RPiCamStillApp &app)
 			std::this_thread::sleep_for(10ms);
 		}
 	}
-	else if (options->zsl)
+	else if (options->Get().zsl)
 		app.ConfigureZsl();
 	else
 		app.ConfigureViewfinder();
@@ -216,7 +218,7 @@ static void event_loop(RPiCamStillApp &app)
 	} af_wait_state = AF_WAIT_NONE;
 	int af_wait_timeout = 0;
 
-	bool want_capture = options->immediate;
+	bool want_capture = options->Get().immediate;
 	for (unsigned int count = 0;; count++)
 	{
 		RPiCamApp::Msg msg = app.Wait();
@@ -248,9 +250,9 @@ static void event_loop(RPiCamStillApp &app)
 			LOG(2, "Viewfinder frame " << count);
 			timelapse_frames++;
 
-			bool timed_out = options->timeout && (now - start_time) > options->timeout.value;
-			bool timelapse_timed_out = options->timelapse &&
-									   (now - timelapse_time) > options->timelapse.value &&
+			bool timed_out = options->Get().timeout && (now - start_time) > options->Get().timeout.value;
+			bool timelapse_timed_out = options->Get().timelapse &&
+									   (now - timelapse_time) > options->Get().timelapse.value &&
 									   timelapse_frames >= TIMELAPSE_MIN_FRAMES;
 
 			if (af_wait_state != AF_WAIT_NONE)
@@ -265,11 +267,11 @@ static void event_loop(RPiCamStillApp &app)
 			else if (timed_out || keypressed || timelapse_timed_out)
 			{
 				// Trigger a still capture, unless we timed out in timelapse or keypress mode
-				if ((timed_out && options->timelapse) || (!keypressed && keypress))
+				if ((timed_out && options->Get().timelapse) || (!keypressed && keypress))
 					return;
 
 				keypressed = false;
-				if (options->af_on_capture)
+				if (options->Get().af_on_capture)
 				{
 					libcamera::ControlList cl;
 					cl.set(libcamera::controls::AfMode, libcamera::controls::AfModeAuto);
@@ -288,20 +290,20 @@ static void event_loop(RPiCamStillApp &app)
 				keypressed = false;
 				af_wait_state = AF_WAIT_NONE;
 				timelapse_time = std::chrono::high_resolution_clock::now();
-				if (!options->zsl)
+				if (!options->Get().zsl)
 				{
 					app.StopCamera();
 					app.Teardown();
 					app.ConfigureStill(still_flags);
 				}
-				if (options->af_on_capture)
+				if (options->Get().af_on_capture)
 				{
 					libcamera::ControlList cl;
 					cl.set(libcamera::controls::AfMode, libcamera::controls::AfModeAuto);
 					cl.set(libcamera::controls::AfTrigger, libcamera::controls::AfTriggerCancel);
 					app.SetControls(cl);
 				}
-				if (!options->zsl)
+				if (!options->Get().zsl)
 					app.StartCamera();
 			}
 			else
@@ -312,28 +314,29 @@ static void event_loop(RPiCamStillApp &app)
 		else if (app.StillStream() && want_capture)
 		{
 			want_capture = false;
-			if (!options->zsl)
+			if (!options->Get().zsl)
 				app.StopCamera();
 			LOG(1, "Still capture image received");
 			save_images(app, completed_request);
-			if (!options->metadata.empty())
+			if (!options->Get().metadata.empty())
 				save_metadata(options, completed_request->metadata);
 			timelapse_frames = 0;
-			if (!options->immediate && (options->timelapse || options->signal || options->keypress))
+			if (!options->Get().immediate &&
+				(options->Get().timelapse || options->Get().signal || options->Get().keypress))
 			{
-				if (!options->zsl)
+				if (!options->Get().zsl)
 				{
 					app.Teardown();
 					app.ConfigureViewfinder();
 				}
-				if (options->af_on_capture && options->afMode_index == -1)
+				if (options->Get().af_on_capture && options->Get().afMode_index == -1)
 				{
 					libcamera::ControlList cl;
 					cl.set(libcamera::controls::AfMode, libcamera::controls::AfModeAuto);
 					cl.set(libcamera::controls::AfTrigger, libcamera::controls::AfTriggerCancel);
 					app.SetControls(cl);
 				}
-				if (!options->zsl)
+				if (!options->Get().zsl)
 					app.StartCamera();
 				af_wait_state = AF_WAIT_NONE;
 			}
@@ -351,8 +354,15 @@ int main(int argc, char *argv[])
 		StillOptions *options = app.GetOptions();
 		if (options->Parse(argc, argv))
 		{
-			if (options->verbose >= 2)
-				options->Print();
+			if (options->Get().verbose >= 2)
+				options->Get().Print();
+
+			if (options->GetPlatform() == Platform::PISP && !options->Get().zsl)
+			{
+				LOG_ERROR("WARNING: Capture will not make use of temporal denoise");
+				LOG_ERROR("         Consider using the --zsl option for best results, for example:");
+				LOG_ERROR("         rpicam-still --zsl -o " << options->Get().output);
+			}
 
 			event_loop(app);
 		}
