@@ -498,22 +498,22 @@ static void create_exif_data(std::vector<libcamera::Span<uint8_t>> const &mem, S
 		}
 
 		// Command-line supplied tags.
-		for (auto &exif_item : options->exif)
+		for (auto &exif_item : options->Get().exif)
 		{
 			LOG(2, "Processing EXIF item: " << exif_item);
 			exif_read_tag(exif, exif_item.c_str());
 		}
 
-		if (options->thumb_quality)
+		if (options->Get().thumb_quality)
 		{
 			// Add some tags for the thumbnail. We put in dummy values for the thumbnail
 			// offset/length to occupy the right amount of space, and fill them in later.
 
-			LOG(2, "Thumbnail dimensions are " << options->thumb_width << " x " << options->thumb_height);
+			LOG(2, "Thumbnail dimensions are " << options->Get().thumb_width << " x " << options->Get().thumb_height);
 			entry = exif_create_tag(exif, EXIF_IFD_1, EXIF_TAG_IMAGE_WIDTH);
-			exif_set_short(entry->data, exif_byte_order, options->thumb_width);
+			exif_set_short(entry->data, exif_byte_order, options->Get().thumb_width);
 			entry = exif_create_tag(exif, EXIF_IFD_1, EXIF_TAG_IMAGE_LENGTH);
-			exif_set_short(entry->data, exif_byte_order, options->thumb_height);
+			exif_set_short(entry->data, exif_byte_order, options->Get().thumb_height);
 			entry = exif_create_tag(exif, EXIF_IFD_1, EXIF_TAG_COMPRESSION);
 			exif_set_short(entry->data, exif_byte_order, 6);
 			ExifEntry *thumb_offset_entry = exif_create_tag(exif, EXIF_IFD_1, EXIF_TAG_JPEG_INTERCHANGE_FORMAT);
@@ -531,11 +531,11 @@ static void create_exif_data(std::vector<libcamera::Span<uint8_t>> const &mem, S
 			// Next create the JPEG for the thumbnail, we need to do this now so that we can
 			// go back and fill in the correct values for the thumbnail offsets/length.
 
-			int q = options->thumb_quality;
+			int q = options->Get().thumb_quality;
 			for (; q > 0; q -= 5)
 			{
-				YUV_to_JPEG((uint8_t *)(mem[0].data()), info, options->thumb_width,
-							options->thumb_height, q, 0, thumb_buffer, thumb_len);
+				YUV_to_JPEG((uint8_t *)(mem[0].data()), info, options->Get().thumb_width, options->Get().thumb_height,
+							q, 0, thumb_buffer, thumb_len);
 				if (thumb_len < 60000) // entire EXIF data must be < 65536, so this should be safe
 					break;
 				free(thumb_buffer);
@@ -595,15 +595,15 @@ void jpeg_save(std::vector<libcamera::Span<uint8_t>> const &mem, StreamInfo cons
 		// YUV422 or YUV420 planar format).
 
 		jpeg_mem_len_t jpeg_len;
-		YUV_to_JPEG((uint8_t *)(mem[0].data()), info, info.width, info.height, options->quality, options->restart,
-					jpeg_buffer, jpeg_len);
+		YUV_to_JPEG((uint8_t *)(mem[0].data()), info, info.width, info.height, options->Get().quality,
+					options->Get().restart, jpeg_buffer, jpeg_len);
 		LOG(2, "JPEG size is " << jpeg_len);
 
 		// Write everything out.
 
 		fp = filename == "-" ? stdout : fopen(filename.c_str(), "w");
 		if (!fp)
-			throw std::runtime_error("failed to open file " + options->output);
+			throw std::runtime_error("failed to open file " + options->Get().output);
 
 		LOG(2, "EXIF data len " << exif_len);
 
